@@ -728,9 +728,9 @@ class ModelBase(object):
         for k, v in self._regs.items():
             this_loss = to_tensor(0.0, requires_grad=True)
             if 'model' in v.signature.inputs:
-                this_loss = v(self._model) if self.training_context['stop_update'] < 1 else to_tensor(0.0, requires_grad=True)
+                reg_weight=v.signature.inputs['reg_weight'].default if 'reg_weight' in v.signature.inputs and hasattr(v.signature.inputs['reg_weight'],'default') else 1e-6
+                this_loss = v(self._model,reg_weight=reg_weight) if self.training_context['stop_update'] < 1 else to_tensor(0.0, requires_grad=True)
             elif 'output' in v.signature.inputs:
-
                 this_loss = try_map_args_and_call(v, self.train_data, self.training_context['data_feed'], self.is_autocast_enabled) if self.training_context[
                                                                                                                                            'stop_update'] < 1 else to_tensor(0.0)
             if not any_abnormal_number(this_loss):
