@@ -22,12 +22,12 @@ from trident.misc.ipython_utils import is_in_ipython, is_in_colab
 from trident.misc.visualization_utils import *
 
 if get_backend() == 'pytorch':
-    from trident.backend.pytorch_backend import try_map_args_and_call, Layer
+    from trident.backend.pytorch_backend import try_map_args_and_call, Layer,Sequential
     from trident.backend.pytorch_ops import to_numpy, arange, cast, clip, sqrt, int_shape, argmax, softmax, ndim, exp, \
         expand_dims
 
 elif get_backend() == 'tensorflow':
-    from trident.backend.tensorflow_backend import try_map_args_and_call, Layer
+    from trident.backend.tensorflow_backend import try_map_args_and_call, Layer,Sequential
     from trident.backend.tensorflow_ops import to_numpy, arange, cast, clip, sqrt, int_shape, zeros_like, ones_like, argmax, softmax, ndim, exp, not_equal, expand_dims
 
 if is_in_ipython() or is_in_colab():
@@ -93,7 +93,7 @@ class TileImageCallback(VisualizationCallbackBase):
         if self.reverse_image_transform is None:
             if len(ctx.get_data_provider()) == 1:
                 self.reverse_image_transform = dataprovider.reverse_image_transform
-                self.reverse_image_transform_funcs = dataprovider.reverse_image_transform_funcs
+                #self.reverse_image_transform_funcs = dataprovider.reverse_image_transform_funcs
 
         if self.include_input:
             input = to_numpy(data[data_feed[model.signature.inputs.key_list[0]]])
@@ -119,7 +119,7 @@ class TileImageCallback(VisualizationCallbackBase):
                         mask = to_numpy(data[ds.symbol])
 
         reverse_image_transform = self.reverse_image_transform
-        reverse_image_transform_funcs = self.reverse_image_transform_funcs
+        reverse_image_transform_funcs = dataprovider.reverse_image_transform_funcs
         if self.include_input and input is not None:
             if len(reverse_image_transform_funcs) > 1:
                 input_arr = []
@@ -322,6 +322,9 @@ class SegTileImageCallback(VisualizationCallbackBase):
                 output_arr = output[:, :, :, 1:2] * argmax(output, axis)
             else:
                 output_arr = (output[:, 1:2, :, :] * argmax(output, axis)).transpose(0, 2, 3, 1)
+        if  ndim(output)==4 and int_shape(output)[1 if get_backend() == 'pytorch' else -1]>4:
+            output  = argmax(output, 1 if get_backend() == 'pytorch' else -1)
+            output_arr = output
 
 
 
@@ -550,7 +553,7 @@ class PrintGradientsCallback(VisualizationCallbackBase):
                         self.last_layer = OrderedDict()
 
                     if len(self.first_layer) == 0 and len(self.last_layer) == 0:
-                        if training_context['current_model'][-1].__class__.__name__ == 'ModuleDict':
+                        if  isinstance(training_context['current_model'],Sequential) and training_context['current_model'][-1].__class__.__name__ == 'ModuleDict':
                             self.is_modulefict = True
                             for k, v in training_context['current_model'][-1].items():
                                 last_layer_name = ''
