@@ -475,6 +475,9 @@ class TextSequenceDataProvider(object):
             raise ValueError("Valid mode should be tuple or dict ")
 
         self.traindata = traindata
+        if  isinstance(traindata,Iterator):
+            if isinstance(traindata.data,TextSequenceDataset):
+                self.sequence_length=traindata.data.sequence_length
         self.testdata = testdata
         self.annotations = {}
 
