@@ -860,7 +860,7 @@ class TextSequenceDataset(Dataset):
     def __init__(self, corpus=None, is_onehot=False, sequence_offset=0, storage_unit='section',section_delimiter='\n\n', stopwords=None, sequence_length: int = 64, sequence_start_at='random',
                  include_segment_ids=False,include_mask_ids=False,object_type=ObjectType.corpus, symbol=None, **kwargs):
         super().__init__(None,symbol=symbol, object_type=object_type, **kwargs)
-        valid_sequence_start_at = ['random', 'slide', 'follow_up', 'section_start']
+        valid_sequence_start_at = ['random', 'slide', 'follow_up', 'section_start','word_start']
         self.sequence_length = sequence_length
         if sequence_start_at in valid_sequence_start_at:
             self.sequence_start_at = sequence_start_at
