@@ -189,7 +189,9 @@ class VisionTransform(Transform):
             trans_boxes = np.concatenate((minxy, maxxy), axis=1)
             trans_boxes[:, 0::2] =clip(trans_boxes[:, 0::2] , 0, ew)
             trans_boxes[:, 1::2] = clip(trans_boxes[:, 1::2],0, eh)
-            if class_info is not None  and class_info.shape[-1]>0:
+            if class_info is not None  and class_info.shape[-1]>0 and (keypoints is None or len(keypoints)==0):
+                trans_boxes = np.concatenate((trans_boxes, class_info), axis=1)
+            elif class_info is not None  and class_info.shape[-1]>0:
                 trans_boxes = np.concatenate((trans_boxes, class_info,keypoints), axis=1)
             return trans_boxes
 
