@@ -310,7 +310,9 @@ class RFBnet(Layer):
 
         location = self.regression_headers[i](x)
         location = location.permute(0, 2, 3, 1).contiguous()
-        location = location.view(location.size(0), -1, 4)
+
+        num_batches = location.size(0)
+        location = location.view(num_batches, -1, self.num_regressors)
 
         return confidence, location
 
