@@ -22,7 +22,7 @@ from trident.backend.tensorspec import ObjectType
 from trident.layers.pytorch_layers import Dense
 
 from trident import context
-
+from trident.backend.model import ModelBase
 from trident.backend.common import dtype as Dtype
 from trident.backend.common import *
 from trident.backend.pytorch_backend import *
@@ -216,7 +216,7 @@ class _ClassificationLoss(Loss):
                 if self.num_classes== len(self.label_statistics):
                     new_shp = [1] * len(int_shape(output))
                     new_shp[1] = len(self.label_statistics)
-                    output = output * clip(to_tensor(np.reshape(self.label_statistics.copy(), tuple(new_shp))), min=1e-7)
+                    output = output * clip(to_tensor(np.reshape(self.label_statistics.copy(), tuple(new_shp)),requires_grad=False), min=1e-7)
 
             output = clip(output, min=1e-7, max=1)
 
@@ -228,7 +228,7 @@ class _ClassificationLoss(Loss):
                 if int_shape(output)[1] == len(self.label_statistics):
                     new_shp = [1] * len(int_shape(output))
                     new_shp[1] = len(self.label_statistics)
-                    output = output - to_tensor(np.reshape(np.log(clip(self.label_statistics.copy(), min=1e-7)), tuple(new_shp)))
+                    output = output - to_tensor(np.reshape(np.log(clip(self.label_statistics.copy(), min=1e-7)), tuple(new_shp)),requires_grad=False)
 
         else:
             self.is_logsoftmax = False
@@ -1230,7 +1230,7 @@ class WingLoss(_PairwiseLoss):
             delta_y - c
         )
 
-        return reduce_mean(losses, [1, 2])
+        return losses
 
 
 class AdaptiveWingLoss(_PairwiseLoss):
@@ -1425,19 +1425,18 @@ def ssim(X, Y, window, data_range: float, use_padding: bool = False):
     return ssim_val, cs
 
 
-class MS_SSIMLoss(_Loss):
+class MS_SSIMLoss(_PairwiseLoss):
     def __init__(self, window_size=11, window_sigma=1.5, data_range=255., channel=3, use_padding=False, weights=None, levels=4, eps=1e-8):
         super(MS_SSIMLoss, self).__init__()
 
         self.window_size = window_size
         self.channel = channel
         window = create_window(window_size, window_sigma, channel)
-        self.register_buffer('window', window.to(get_device()))
+        self.window=window.to(get_device())
         self.data_range = data_range
         self.use_padding = use_padding
         self.eps = eps
-        window = create_window(window_size, window_sigma, channel)
-        self.register_buffer('window', window.to(get_device()))
+
 
         if weights is None:
             weights = [0.0448, 0.2856, 0.3001, 0.2363, 0.1333]
@@ -1447,7 +1446,7 @@ class MS_SSIMLoss(_Loss):
             weights = weights[:levels]
             weights = weights / weights.sum()
 
-        self.register_buffer('weights', weights.to(get_device()))
+        self.weights=weights.to(get_device())
 
     def ms_ssim(self, X, Y):
         '''
@@ -1491,8 +1490,8 @@ class MS_SSIMLoss(_Loss):
         # In this file's image training demo. I feel the old ms-ssim more better. So I keep use old ms-ssim op.
         return ms_ssim_val
 
-    def forward(self, output, target) -> 'loss':
-        return self.ms_ssim(output, target).mean()
+    def calculate_loss(self, output, target, **kwargs):
+        return self.ms_ssim(output, target)
 
 
 class IoULoss(_ClassificationLoss):
@@ -1956,7 +1955,7 @@ class StyleLoss(nn.Module):
 class PerceptionLoss(_PairwiseLoss):
     def __init__(self, net, reduction="mean"):
         super(PerceptionLoss, self).__init__()
-        if issubclass(net, get_class('Model', 'trident.optims.pytorch_trainer')):
+        if isinstance(net, ModelBase):
             net = net.model
         self.ref_model = net
         self.ref_model.trainable = False
@@ -1967,7 +1966,7 @@ class PerceptionLoss(_PairwiseLoss):
                 module.keep_output = True
 
         self.reduction = reduction
-        self.to(_device)
+        self.ref_model.to(_device)
 
     def vgg_preprocess(self, img):
         return ((img + 1) / 2) * to_tensor([[0.485, 0.456, 0.406]]).unsqueeze(-1).unsqueeze(-1) + to_tensor([[0.229, 0.224, 0.225]]).unsqueeze(-1).unsqueeze(-1)
@@ -1986,10 +1985,12 @@ class PerceptionLoss(_PairwiseLoss):
         target_features = OrderedDict()
         output_features = OrderedDict()
         _ = self.ref_model(self.vgg_preprocess(output))
+        #_ = self.ref_model(output)
         for item in self.layer_name_mapping.values():
             output_features[item] = getattr(self.ref_model, item).output
 
-        _ = self.ref_model(self.preprocess(target))
+        _ = self.ref_model(self.vgg_preprocess(target))
+        #_ = self.ref_model(target)
         for item in self.layer_name_mapping.values():
             target_features[item] = getattr(self.ref_model, item).output.detach()
 
@@ -1997,9 +1998,9 @@ class PerceptionLoss(_PairwiseLoss):
         num_filters = 0
         for i in range(len(self.layer_name_mapping)):
             b, c, h, w = output_features.value_list[i].shape
-            loss += ((output_features.value_list[i] - target_features.value_list[i]) ** 2).sum() / (h * w)
-            num_filters += c
-        return loss / (output.size(0) * num_filters)
+            loss=loss+ ((output_features.value_list[i] - target_features.value_list[i]) ** 2).sum()/(c*h*w)
+
+        return loss
 
 
 class EdgeLoss(_Loss):
