@@ -377,15 +377,46 @@ class TrainingPlan(object):
                 available_items.remove("")
 
             for inp in trainingitem.signature.inputs.key_list:
-                data_feed[inp] = None
+                if existing_data_feed is not None and isinstance(existing_data_feed,OrderedDict) and inp in existing_data_feed:
+                    data_feed[inp] = existing_data_feed[inp]
+                elif inp in data_symbols:
+                    data_feed[inp]=inp
+                    available_items.remove(inp)
+                elif inp in unpair_symbols:
+                    data_feed[inp]=inp
+                    available_items.remove(inp)
+                else:
+                    data_feed[inp] = None
             for k, v in trainingitem._losses.items():
                 for inp in v.signature.inputs.key_list:
-                    data_feed[inp] = None
+                    if existing_data_feed is not None and inp in existing_data_feed :
+                        data_feed[inp] = existing_data_feed[inp]
+                    elif inp in data_feed and  data_feed[inp] is not None:
+                        pass
+                    elif inp in label_symbols:
+                        data_feed[inp] = inp
+                        available_items.remove(inp)
+                    elif inp in unpair_symbols:
+                        data_feed[inp] = inp
+                        available_items.remove(inp)
+                    else:
+                        data_feed[inp] = None
             for k, v in trainingitem._metrics.items():
                 for inp in v.signature.inputs.key_list:
-                    data_feed[inp] = None
+                    if existing_data_feed is not None and inp in existing_data_feed:
+                        data_feed[inp] = existing_data_feed[inp]
+                    elif inp in data_feed and data_feed[inp] is not None:
+                        pass
+                    elif inp in label_symbols:
+                        data_feed[inp] = inp
+                        available_items.remove(inp)
+                    elif inp in unpair_symbols:
+                        data_feed[inp] = inp
+                        available_items.remove(inp)
+                    else:
+                        data_feed[inp] = None
 
-            if 'x' in data_feed:
+            if 'x' in data_feed and data_feed['x'] is None:
                 if len(data_symbols) == 1 and data_symbols[0] in available_items:
                     data_feed['x'] = data_symbols[0]
                     available_items.remove(data_symbols[0])
@@ -396,38 +427,40 @@ class TrainingPlan(object):
                 elif 'input' in available_items:
                     data_feed['x'] = 'input'
                     available_items.remove('input')
+            if len([item for item  in data_feed.value_list if item is None])==0:
+                pass
+            else:
 
+                if len(trainingitem.signature.inputs) == len(data_symbols) == 1:
+                    # if trainingitem.signature.inputs.value_list[0].shape.is_compatible_with(data_provider.traindata.data.element_spec.shape):
+                    data_feed[trainingitem.signature.inputs.key_list[0]] = data_provider.traindata.data.symbol
+                    if data_provider.traindata.data.symbol in available_items:
+                        available_items.remove(data_provider.traindata.data.symbol)
 
-            if len(trainingitem.signature.inputs) == len(data_symbols) == 1:
-                # if trainingitem.signature.inputs.value_list[0].shape.is_compatible_with(data_provider.traindata.data.element_spec.shape):
-                data_feed[trainingitem.signature.inputs.key_list[0]] = data_provider.traindata.data.symbol
-                if data_provider.traindata.data.symbol in available_items:
-                    available_items.remove(data_provider.traindata.data.symbol)
-
-            if len(trainingitem.signature.outputs) == 1 and len(label_symbols) == 0:
-                data_feed[trainingitem.signature.outputs.key_list[0].replace("output", "target").replace("student", "teacher")] = data_provider.traindata.data.symbol
-                if data_provider.traindata.label.symbol in available_items:
+                if len(trainingitem.signature.outputs) == 1 and len(label_symbols) == 0:
+                    data_feed[trainingitem.signature.outputs.key_list[0].replace("output", "target").replace("student", "teacher")] = data_provider.traindata.data.symbol
+                    if data_provider.traindata.label.symbol in available_items:
+                        available_items.remove(data_provider.traindata.label.symbol)
+                elif len(trainingitem.signature.outputs) == len(label_symbols) == 1:
+                    data_feed[trainingitem.signature.outputs.key_list[0].replace("output", "target").replace("student", "teacher")] = data_provider.traindata.label.symbol
                     available_items.remove(data_provider.traindata.label.symbol)
-            elif len(trainingitem.signature.outputs) == len(label_symbols) == 1:
-                data_feed[trainingitem.signature.outputs.key_list[0].replace("output", "target").replace("student", "teacher")] = data_provider.traindata.label.symbol
-                available_items.remove(data_provider.traindata.label.symbol)
 
-            for out in trainingitem.signature.outputs.key_list:  # fill the data_feed by key
-                if out in available_items:  # output=putput
-                    data_feed[out] = out
-                    available_items.remove(out)
+                for out in trainingitem.signature.outputs.key_list:  # fill the data_feed by key
+                    if out in available_items:  # output=putput
+                        data_feed[out] = out
+                        available_items.remove(out)
 
-            if 'target' in data_feed:
-                if len(label_symbols) == 1 and label_symbols[0] in available_items:
-                    data_feed['target'] = label_symbols[0]
-                    available_items.remove(label_symbols[0])
+                if 'target' in data_feed:
+                    if len(label_symbols) == 1 and label_symbols[0] in available_items:
+                        data_feed['target'] = label_symbols[0]
+                        available_items.remove(label_symbols[0])
 
-            for key in data_feed.keys():
-                if data_feed[key] == None and key in available_items:
-                    data_feed[key] = key
-                    available_items.remove(key)
-                elif data_feed[key] == None:
-                    data_feed[key] = key
+                for key in data_feed.keys():
+                    if data_feed[key] == None and key in available_items:
+                        data_feed[key] = key
+                        available_items.remove(key)
+                    elif data_feed[key] == None:
+                        data_feed[key] = key
 
                 # elif out.replace("output","target").replace("student","teacher") in available_items:
                 #     data_feed[out] =out.replace("output","target").replace("student","teacher")
