@@ -662,13 +662,18 @@ class ActorCriticPolicy(MuiltiNetwork):
 
     def calculate_gae(self, values, next_values, rewards, dones, gamma: float, gae_lambda: float, **kwargs):
         try:
-            R = [0] * len(rewards)
-            delta = to_tensor(rewards) + next_values * gamma - values
-            m = (1.0 - dones) * (gamma * gae_lambda)
+            R = []
+            # delta = to_tensor(rewards) + next_values * gamma - values
+            # m = (1.0 - dones) * (gamma * gae_lambda)
             gae = 0.0
-            for i in range(len(rewards) - 1, -1, -1):
-                gae = delta[i] + m[i] * gae
-                R[i] = gae
+            # for i in range(len(rewards) - 1, -1, -1):
+            #     gae = delta[i] + m[i] * gae
+            #     R[i] = gae+values[i]
+            for value,next_value, reward, done in list(zip(values,next_values, rewards, dones))[::-1]:
+                gae = gae * gamma * gae_lambda
+                gae = gae + reward +gamma * next_value.detach() * (1 - done) - value.detach()
+                R.insert(0,gae + value)
+
             R = stack(R, axis=0)
             return R
         except Exception as e:
@@ -1399,8 +1404,8 @@ class PPOPolicy(ActorCriticPolicy):
             last_done = done_pool[-1]
             last_next_value = 0.0 if last_done else self.critic.model(state_pool[-1])[:, action_pool[-1]].item()
             running_add = last_next_value
-            R = []
-            V = []
+            DiscountedReturns = []
+            Values = []
             Ratio1 = []
             Ratio2 = []
             LogProbs = []
@@ -1414,31 +1419,35 @@ class PPOPolicy(ActorCriticPolicy):
 
             for i in reversed(range(len(reward_pool))):
                 running_add = running_add * self.gamma + reward_pool[i]
-                R.insert(0, running_add)
+                DiscountedReturns.insert(0, running_add)
                 value = self.critic.model(state_pool[i])[:, action_pool[i]]
-                V.insert(0, value)
+                Values.insert(0, value)
 
-            R = to_tensor(R).detach()
+            DiscountedReturns = to_tensor(DiscountedReturns).detach()
+            #R=(R-R.mean())/(R.std()+1e-7)
 
-            V_ = V.copy()[1:]
-            V_.append(to_tensor(running_add))
-            V = concate(V, axis=0)
-            V_ = concate(V_, axis=0)
-            clipped_value = where(V > V_ * 1.2, V_ * 1.2, V)
-            clipped_value = where(clipped_value < V_ * 0.8, V_ * 0.8, clipped_value)
+            NextValues = Values.copy()[1:]
+            NextValues.append(to_tensor(running_add))
+            PrevValues=Values.copy()[:-1]
+            PrevValues.insert(0,Values[0])
+
+            Values = concate(Values, axis=0)
+            NextValues = concate(NextValues, axis=0)
+            PrevValues = concate(PrevValues, axis=0)
+            clipped_value = PrevValues+clip(Values-PrevValues,-0.2,0.2)
 
             done_pool = to_tensor(done_pool).float().detach()
             advantages = None
             if self.use_gae:
 
-                gaes = self.calculate_gae(V, V_, R, done_pool, self.gamma, self.gae_lambda)
-                clipped_gaes = self.calculate_gae(clipped_value, V_, R, done_pool, self.gamma, self.gae_lambda)
-                advantages = max(gaes, clipped_gaes)
+                gaes = self.calculate_gae(Values, NextValues, DiscountedReturns, done_pool, self.gamma, self.gae_lambda)
+                clipped_gaes = self.calculate_gae(clipped_value, NextValues, DiscountedReturns, done_pool, self.gamma, self.gae_lambda)
+                advantages = max(gaes, clipped_gaes).detach()-Values
                 if self.normalize_advantages:
                     # only divide std
                     advantages = advantages / (advantages.std() + 1e-5)
             else:
-                advantages = max(V - R, clipped_value - R)
+                advantages = max(DiscountedReturns.detach()-Values, DiscountedReturns.detach()-clipped_value)
 
             critic_loss = (advantages ** 2).mean()
 
@@ -1463,19 +1472,19 @@ class PPOPolicy(ActorCriticPolicy):
 
             if self.done == True:
 
-                actor_loss = -1 * (LogProbs * (advantages.detach())).mean()
+                actor_loss = -1 * (LogProbs * (advantages)).mean()
             else:
-                surr1 = Ratio1 * (advantages.detach())
-                surr2 = Ratio2 * (advantages.detach())
+                surr1 = Ratio1 * (advantages)
+                surr2 = Ratio2 * (advantages)
                 actor_loss = None
                 actor_loss = (-1 * minimum(surr1, surr2)).mean()
 
-            return critic_loss, actor_loss, advantages, R, V
+            return critic_loss, actor_loss, advantages, DiscountedReturns, Values
         except Exception as e:
             print(e)
 
     def save_or_sync_weights(self):
-        if self.current_epoch <= 1 or (self.train_steps) % self.accumulation_steps  == 0:
+        if self.train_steps <= 1 or (self.train_steps) % (5*self.accumulation_steps)  == 0:
             self.save_model(save_path=self.training_context['save_path'])
             self.critic.model.to(get_device())
             self.actor.model.to(get_device())
@@ -1514,7 +1523,7 @@ class PPOPolicy(ActorCriticPolicy):
                             self.train_steps += 1
                             self.current_batch += 1
                             self.steps = self.train_steps
-
+                            accumulate_grads = self.training_context['steps'] % self.accumulation_steps != 0
                             # self.do_on_batch_start()
 
                             batch_data = self.memory.get_samples(builtins.min(64, len(self.memory)))
@@ -1531,13 +1540,14 @@ class PPOPolicy(ActorCriticPolicy):
                             self.critic.current_loss = batch_critic_loss
                             self.critic.batch_loss_history.collect('critic_loss', self.train_steps, batch_critic_loss.copy().detach())
                             self.critic.batch_metric_history.collect('critic_rmse', self.train_steps, (batch_critic_loss.copy().detach()).sqrt())
-                            self.update_critic(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', False, accumulate_grads=False)
+                            self.update_critic(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', False, accumulate_grads=accumulate_grads)
                             self.critic.current_loss = 0.0
                             batch_actor_loss = to_tensor(0.0)
                             for j in range(len(batch_state)):
-                                old_batch_probs = self.old_actor(to_tensor(batch_state[j]))
-                                old_m = Categorical(probs=old_batch_probs)
+                                #old_batch_probs = self.old_actor(to_tensor(batch_state[j]))
+                                #old_m = Categorical(probs=old_batch_probs)
                                 old_batch_probs = batch_log_prob[j].long().detach()
+                                #old_batch_probs=old_m.log_prob(to_tensor(batch_action[j]).long())
                                 new_batch_probs = self.actor(to_tensor(batch_state[j]))
                                 new_m = Categorical(probs=new_batch_probs)
                                 new_log_prob = new_m.log_prob(to_tensor(batch_action[j]).long())
@@ -1552,8 +1562,9 @@ class PPOPolicy(ActorCriticPolicy):
                             self.actor.batch_loss_history.collect('actor_loss', self.train_steps, batch_actor_loss.copy() / len(batch_state))
                             # self.actor.batch_loss_history.collect('entropy_loss', self.train_steps, (-0.01 * entropy_loss).copy().detach() / (len(state_pool) - 1))
 
-                            self.update_actor(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', False, accumulate_grads=False)
+                            self.update_actor(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', False, accumulate_grads=accumulate_grads)
                             self.actor.current_loss = 0.0
+                            self.save_or_sync_weights()
 
                         except Exception as e:
                             print(e)
@@ -1563,7 +1574,7 @@ class PPOPolicy(ActorCriticPolicy):
                     # self.actor.print_batch_progress(8 if done else 2)
 
                 if len(reward_pool) > 3:
-
+                    accumulate_grads = self.training_context['steps'] % self.accumulation_steps != 0
                     try:
                         self.train_steps += 1
                         self.current_batch += 1
@@ -1577,7 +1588,7 @@ class PPOPolicy(ActorCriticPolicy):
                         self.critic.batch_loss_history.collect('critic_loss', self.train_steps, critic_loss.copy().detach())
                         self.critic.batch_metric_history.collect('critic_rmse', self.train_steps, critic_loss.copy().detach().sqrt())
 
-                        self.update_critic(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', self.done, accumulate_grads=False)
+                        self.update_critic(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', self.done, accumulate_grads=accumulate_grads)
                         # self.critic.print_batch_progress(1)
                         self.critic.current_loss = 0
 
@@ -1598,12 +1609,12 @@ class PPOPolicy(ActorCriticPolicy):
                         self.actor.batch_loss_history.collect('actor_loss', self.train_steps, actor_loss.copy().detach())
                         # self.actor.batch_loss_history.collect('entropy_loss', self.train_steps, (-0.01 * entropy_loss).copy().detach() / (len(state_pool) - 1))
 
-                        self.update_actor(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', self.done, accumulate_grads=False)
+                        self.update_actor(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', self.done, accumulate_grads=accumulate_grads)
                         self.actor.current_loss = 0.0
                         # self.actor.print_batch_progress(1)
-                        if self.done == True:
-                            self.save_or_sync_weights()
-                            # self.memory.reset()
+                        self.save_or_sync_weights()
+
+
 
                         for idx in list(range(len(state_pool)))[::-1]:
                             if self.rollout.reward[idx] != 0:
